from ast import keyword
import datetime
from joblib import Parallel
import nltk
from newspaper import Article  # for parsing Reuters articles
import numpy as np
from dataclasses import dataclass
import requests
import csv
import pickle
import json
import os
from joblib import Parallel, delayed  # for parallel processing
from tqdm import tqdm



def convert_link_to_article(link, ticker):
    if link == "https://www.reuters.com/article/health-coronavirus-regeneron-pharms-anti/eu-assesses-regeneron-roche-covid-19-antibody-cocktail-for-authorisation-idUSL4N2R72J9":
        print(f"Converting {link} for {ticker}")
    article = Article(link)  # Instantiate the Article() object
    article.download()  # Download the article
    article.parse()  # Parse the article for data
    article.nlp()  # Run Natural Language Processing on the article

    # if published date is not available, use the date of the article
    if article.publish_date == None:
        print(f"No publish date for {link}, ticker: {ticker}")

    return NewsStory(ticker, article.title,  link, article.authors, article.publish_date.strftime("%x"), article.text, article.keywords, article.summary)


@dataclass
class NewsStory:
    stock_ticker: str
    title: str
    url: str
    authors: str
    publish_date: str
    text: str
    keywords: str
    summary: str

    def __repr__(self):
        return f"Ticker: {self.stock_ticker} \nTitle: {self.title} \nURL: {self.url} \nAuthors: {self.authors} \nPublish Date: {self.publish_date} \nText: {self.text} \nKeywords: {self.keywords} \nSummary: {self.summary}\n"

    def __hash__(self):
        return hash(self.url)


def news_inCSV(file):
    # takes in csv file in the format of:
    # ,text,link
    # returns a list of NewsStory objects
    ticker = file.split(".")[1].split("/")[-1]
    newsStories = []
    with open(file, 'r') as csvfile:
        csvreader = csv.reader(csvfile)
        next(csvreader)
        for row in csvreader:
            newsStories.append(
                NewsStory(ticker, row[1], row[2], "", "", "", "", ""))
    # remove duplicates
    newsStories = list(set(newsStories))
    return newsStories


def enrichCSV(file):
    print(f"Enriching {file}")
    newsStories = []
    newsStoriesEnriched = []
    newsStories = news_inCSV(file)
    # run the scraper on each article
    for newsStory in tqdm(newsStories):
        newsStory = convert_link_to_article(
            newsStory.url, newsStory.stock_ticker)
        newsStoriesEnriched.append(newsStory)
    return newsStoriesEnriched


def writeNewsToJSON(newsStories, file_name):
    file_name = "./reuters_data_enriched/" + file_name + ".json"
    with open(file_name, 'w') as outfile:
        outfile.write(json.dumps(
            newsStories, default=lambda o: o.__dict__, sort_keys=True, indent=4))


def enrichAndWriteNewsToJSON(file):
    enriched = enrichCSV("./reuters_data/" + file)
    writeNewsToJSON(enriched, file.split(".")[0])
    # write empty file to ./done/
    with open("./done/" + file, 'w') as outfile:
        outfile.write("")


if __name__ == "__main__":

    # for all files in the reuters_data folder
    files = os.listdir("./reuters_data")
    print("number of files total: ", len(files))
    # remove files that have already been processed
    files = [file for file in files if file not in os.listdir("./done")]
    print("number of files to process: ", len(files))

    # Parallel(2, "loky", verbose=20)(
    #    delayed(enrichAndWriteNewsToJSON)(file) for file in files)

    for file in files:
        enrichAndWriteNewsToJSON(file)
