from selenium import webdriver
import time
import pandas as pd

# function based on https://github.com/miguelaenlle/Reuters-Scraping
def get_data_for_stock(stock):
    fireFoxOptions = webdriver.FirefoxOptions()
    fireFoxOptions.add_argument("--headless")
    driver = webdriver.Firefox(options=fireFoxOptions)

    try:
        # Search Reuters for {stock}
        driver.get("https://www.reuters.com/search/news?blob={}".format(stock))
        time.sleep(2)  # Wait for the page to load

        # Reuters should query the company if they have written articles on it.
        # If they haven't, an error will be thrown and no files will be outputted
        # This line gets the company they queried's element which contains
        # the stock's name and URL to Reuters's page on the stock
        text = driver.find_element_by_xpath(
            "/html/body/div[4]/section[2]/div/div[1]/div[3]/div/div/div/div[1]/a"
        ).text

        # {condition} will determine if the stock queried is actually
        # the stock that we're trying to get articles on
        condition = False

        # Reuters will format the element's text in 2 ways:

        # {company name} ({ticker}.{some additional text})
        # e.g.
        # Apple Inc (AAPL.OQ)
        # or
        # {company name} ({ticker})
        # e.g.
        # Alcoa Corp (AA)

        # The following lines of code will filter down the element's text
        # to just get {ticker}
        # e.g.
        # Apple Inc (AAPL.OQ) --> AAPL
        # Aloca Corp (AA)     --> AA
        if "." in text:  # Check if a period is in the text
            # Check if {ticker}.upper() == {stock}.upper()
            if text[text.find("(") + 1: text.find(".")].upper() == stock.upper():
                # {condition} = True means that the queried stock is a match
                condition = True
        else:  # If there is no period
            # Check if {ticker}.upper() == {stock}.upper()
            if text[text.find("(") + 1: text.find(")")].upper() == stock.upper():
                # {condition} = True means that the queried stock is a match
                condition = True

        if condition:  # If {stock} has been found in Reuters, continue

            # Click the element's link, going to Reuters's
            driver.find_element_by_xpath(
                "/html/body/div[4]/section[2]/div/div[1]/div[3]/div/div/div/div[1]/a"
            ).click()
            time.sleep(0.5)  # Let the stock's Reuters page load

            # Go to the "News" section of the stock's Reuters page
            driver.find_element_by_xpath(
                "/html/body/div[1]/div/div[3]/div/div/nav/div[1]/div/div/ul/li[2]/button"
            ).click()
            time.sleep(5)

            # The next segment will scroll down to the bottom of the "News"
            # page of the stock's Reuters page
            SCROLL_PAUSE_TIME = (
                0.5  # This is how much time it waits before scrolling to
            )
            # the bottom of the page again

            # Get the last height of the page
            last_height = driver.execute_script(
                "return document.body.scrollHeight")
            while True:
                # Scroll to the bottom of the page
                driver.execute_script(
                    "window.scrollTo(0, document.body.scrollHeight);")

                # Wait for more content to load
                time.sleep(SCROLL_PAUSE_TIME)

                # Get the current height of the page
                new_height = driver.execute_script(
                    "return document.body.scrollHeight")
                if new_height == last_height:
                    # If the current height of the page is the same as it was before,
                    # break the script because there is no more content to load.
                    break

                last_height = new_height  # Get the previous height of the page
            i = 1  # Article index number starts at one in the HTML
            datas = []  # Put all of the data in here
            # Tol is the amount of times an error has been thrown (for this stock news page).
            tol = 0
            # When it hits 3, it's confirmed that all articles for this stock
            # have been queried.

            # The amount of articles on the page is unknown, so a while loop is
            # used to iterate until no more new articles are found
            while True:
                try:
                    # This is the xpath for the title of the article
                    xpath = "/html/body/div[1]/div/div[4]/div[1]/div/div/div/div[2]/div[{}]/div/a".format(
                        i
                    )
                    i += 1
                    # An error will be thrown if there are no more articles left because
                    # the driver won't be able to find the non-existent next article.

                    header = driver.find_element_by_xpath(
                        xpath
                    ).text  # Get the article's header text
                    link = driver.find_element_by_xpath(xpath).get_attribute(
                        "href"
                    )  # Get the link of the article

                    # The links are processed by a seperate script because
                    # less threads have to be devoted to

                    datas.append([header, link])
                except Exception as e:
                    tol += 1  # Increase the error tally by 1
                    time.sleep(30)

                    if (
                        tol > 2
                    ):  # The script will only break if 3 errors in a row are thrown to confirm
                        # it's actually found all the articles
                        break

            # Compile the list of headers and links into a pandas DataFrame
            datas = pd.DataFrame(datas, columns=["text", "link"])
            # Export the data to the reuters data folder under the name {stock}.csv
            datas.to_csv("reuters_data/{}.csv".format(stock))
        else:
            print("{} not found in Reuters".format(stock))
            # write the stock to the not found file
            with open("not_found.txt", "a") as f:
                f.write(stock + "\n")

        # Stop the driver
        driver.quit()
    except Exception as e:
        # Make this worker wait a bit before killing incase Reuters.com
        time.sleep(30)
        driver.quit()

if __name__ == "__main__":

    # for all symbols in symbols.txt
    # but not in ./reuters_data/SYMBOL.csv

